import sets

import pygame
from pygame.locals import *

from pgu import tilevid, timer, ani

CODE_START = 1
CODE_LADDER = 3

TILE_CLEAR = 0
TILE_LADDER = 2
TILE_LADDER_PLATFORM = 3
CLEAR_TILES = sets.Set([TILE_CLEAR] + range(48, 80))

class Game:
    def __init__(self):
        g = self.game = tilevid.Tilevid()

        g.screen = pygame.display.set_mode((640, 480), SWSURFACE)

        cdata = {
            #TILE_CLEAR: ('player', self.player_fall, {}),
        }
        for tileid in range(16, 48):    # 1st to 3rd rows
            cdata[tileid] = ('player', self.tile_block, {})

        g.tga_load_tiles('data/sotc/level1-tiles.tga', (32, 32), cdata)
        g.tga_load_level('data/sotc/level1.tga', 1)
        g.bounds = pygame.Rect(32, 32, (len(g.tlayer[0])-2)*32,
            (len(g.tlayer)-2)*32)

        ani.ani_load(g, 'player', pygame.image.load('data/sotc/player.tga'),
            (32, 32), (8, 8, 16, 24), [4, 3])

        for y, row in enumerate(g.clayer):
            for x, cell in enumerate(row):
                if cell == CODE_START:
                    player_pos = (x, y)
        pos = g.tile_to_screen(player_pos)
        s = tilevid.Sprite(g.images['player.2.1'], pos + (32, 32))
        g.sprites.append(s)
        s.loop = self.player_loop
        s.groups = g.string2groups('player')
        g.player = s

    def player_loop(self, game, player_sprite):
        keys = pygame.key.get_pressed()

        cx, cy = player_sprite.rect.center
        tx, ty = self.game.screen_to_tile((cx, cy))
        tile = self.game.tlayer[ty][tx]

        dx,dy = 0,0
        if keys[K_LEFT]: dx-=1
        if keys[K_RIGHT]: dx+=1
        if tile in (TILE_LADDER, TILE_LADDER_PLATFORM):
            if keys[K_UP]: dy-=1
            if keys[K_DOWN]: dy+=1
        elif tile in CLEAR_TILES:
            dy += 1

        player_sprite.rect.x += dx*5
        player_sprite.rect.y += dy*5
        player_sprite.rect.clamp_ip(game.view)

    def tile_block(self, game, t, a):
        if (a._rect.bottom <= t._rect.top and a.rect.bottom > t.rect.top):
            a.rect.bottom = t.rect.top
        if (a._rect.right <= t._rect.left and a.rect.right > t.rect.left):
            a.rect.right = t.rect.left
        if (a._rect.left >= t._rect.right and a.rect.left < t.rect.right):
            a.rect.left = t.rect.right
        if (a._rect.top >= t._rect.bottom and a.rect.top < t.rect.bottom):
            a.rect.top = t.rect.bottom

    def run(self): 
        self.game.quit = 0
        
        self.game.paint(self.game.screen)
        pygame.display.flip()

        t = timer.Timer(25)
        while not self.game.quit:
            for e in pygame.event.get():
                if e.type is QUIT: self.game.quit = 1
                if e.type is KEYDOWN and e.key == K_ESCAPE: self.game.quit = 1

            self.game.loop()
            pygame.display.update(self.game.update(self.game.screen))
            t.tick()

Game().run()

